﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Vts.Gui.Silverlight.View
{
    /// <summary>
    /// A class for mapping grayscale intensity to a color via lookup tables
    /// </summary>
    public class Colormap
    {
        private readonly ColormapLUT lut;

        private readonly byte[] _RedByte;
        private readonly byte[] _GreenByte;
        private readonly byte[] _BlueByte;

        public Colormap(ColormapType colormapType)
        {
            //bool UseInterpolation = false;
            int colormapSize = 256;

            this._RedByte = new byte[colormapSize];
            this._GreenByte = new byte[colormapSize];
            this._BlueByte = new byte[colormapSize];

            switch (colormapType)
            {
                case ColormapType.Hot:
                    lut = new HotColormapLUT();
                    break;
                case ColormapType.Jet:
                    lut = new JetColormapLUT();
                    break;
                case ColormapType.HSV:
                    lut = new HSVColormapLUT();
                    break;
                case ColormapType.Copper:
                    lut = new CopperColormapLUT();
                    break;
                case ColormapType.Binary:
                    lut = new BinaryColormapLUT();
                    break;
                case ColormapType.Bone:
                    lut = new BoneColormapLUT();
                    break;
                case ColormapType.Gray:
                default:
                    lut = new GrayColormapLUT();
                    break;
            }

            int mappingRatio = colormapSize / lut.Length;
            try
            {
                for (int j = 0; j < colormapSize; j++)
                {
                    this._RedByte[j] = (byte)(255 * lut.Red[j / mappingRatio]);
                    this._GreenByte[j] = (byte)(255 * lut.Green[j / mappingRatio]);
                    this._BlueByte[j] = (byte)(255 * lut.Blue[j / mappingRatio]);
                }
            }
            catch (IndexOutOfRangeException e) { Console.WriteLine(e); }
        }

        public byte[] RedByte { get { return _RedByte; } }
        public byte[] GreenByte { get { return _GreenByte; } }
        public byte[] BlueByte { get { return _BlueByte; } }

        #region Nested ColormapLUT Classes

        internal abstract class ColormapLUT
        {
            #region Properties
            public double[] Red
            {
                get { return this._Red; }
            }
            protected readonly double[] _Red;

            public double[] Green
            {
                get { return this._Green; }
            }
            protected readonly double[] _Green;

            public double[] Blue
            {
                get { return this._Blue; }
            }
            protected readonly double[] _Blue;

            public int Length
            {
                get
                {
                    if (Red != null)
                        return Red.Length;
                    else
                        return 0;
                }
            }
            #endregion

            #region Constructor
            public ColormapLUT(double[] R, double[] G, double[] B)
            {
                this._Red = R;
                this._Green = G;
                this._Blue = B;
            }
            #endregion
        }
        internal class HotColormapLUT : ColormapLUT
        {
            #region Colormap Definitions
            static double[] R = new double[]{
0.010417,
0.020833,
 0.03125,
0.041667,
0.052083,
  0.0625,
0.072917,
0.083333,
 0.09375,
 0.10417,
 0.11458,
   0.125,
 0.13542,
 0.14583,
 0.15625,
 0.16667,
 0.17708,
  0.1875,
 0.19792,
 0.20833,
 0.21875,
 0.22917,
 0.23958,
    0.25,
 0.26042,
 0.27083,
 0.28125,
 0.29167,
 0.30208,
  0.3125,
 0.32292,
 0.33333,
 0.34375,
 0.35417,
 0.36458,
   0.375,
 0.38542,
 0.39583,
 0.40625,
 0.41667,
 0.42708,
  0.4375,
 0.44792,
 0.45833,
 0.46875,
 0.47917,
 0.48958,
     0.5,
 0.51042,
 0.52083,
 0.53125,
 0.54167,
 0.55208,
  0.5625,
 0.57292,
 0.58333,
 0.59375,
 0.60417,
 0.61458,
   0.625,
 0.63542,
 0.64583,
 0.65625,
 0.66667,
 0.67708,
  0.6875,
 0.69792,
 0.70833,
 0.71875,
 0.72917,
 0.73958,
    0.75,
 0.76042,
 0.77083,
 0.78125,
 0.79167,
 0.80208,
  0.8125,
 0.82292,
 0.83333,
 0.84375,
 0.85417,
 0.86458,
   0.875,
 0.88542,
 0.89583,
 0.90625,
 0.91667,
 0.92708,
  0.9375,
 0.94792,
 0.95833,
 0.96875,
 0.97917,
 0.98958,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1};

            static double[] G = new double[]{

       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
0.010417,
0.020833,
 0.03125,
0.041667,
0.052083,
  0.0625,
0.072917,
0.083333,
 0.09375,
 0.10417,
 0.11458,
   0.125,
 0.13542,
 0.14583,
 0.15625,
 0.16667,
 0.17708,
  0.1875,
 0.19792,
 0.20833,
 0.21875,
 0.22917,
 0.23958,
    0.25,
 0.26042,
 0.27083,
 0.28125,
 0.29167,
 0.30208,
  0.3125,
 0.32292,
 0.33333,
 0.34375,
 0.35417,
 0.36458,
   0.375,
 0.38542,
 0.39583,
 0.40625,
 0.41667,
 0.42708,
  0.4375,
 0.44792,
 0.45833,
 0.46875,
 0.47917,
 0.48958,
     0.5,
 0.51042,
 0.52083,
 0.53125,
 0.54167,
 0.55208,
  0.5625,
 0.57292,
 0.58333,
 0.59375,
 0.60417,
 0.61458,
   0.625,
 0.63542,
 0.64583,
 0.65625,
 0.66667,
 0.67708,
  0.6875,
 0.69792,
 0.70833,
 0.71875,
 0.72917,
 0.73958,
    0.75,
 0.76042,
 0.77083,
 0.78125,
 0.79167,
 0.80208,
  0.8125,
 0.82292,
 0.83333,
 0.84375,
 0.85417,
 0.86458,
   0.875,
 0.88542,
 0.89583,
 0.90625,
 0.91667,
 0.92708,
  0.9375,
 0.94792,
 0.95833,
 0.96875,
 0.97917,
 0.98958,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1};

            static double[] B = new double[]{     
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
0.015625,
 0.03125,
0.046875,
  0.0625,
0.078125,
 0.09375,
 0.10938,
   0.125,
 0.14063,
 0.15625,
 0.17188,
  0.1875,
 0.20313,
 0.21875,
 0.23438,
    0.25,
 0.26563,
 0.28125,
 0.29688,
  0.3125,
 0.32813,
 0.34375,
 0.35938,
   0.375,
 0.39063,
 0.40625,
 0.42188,
  0.4375,
 0.45313,
 0.46875,
 0.48438,
     0.5,
 0.51563,
 0.53125,
 0.54688,
  0.5625,
 0.57813,
 0.59375,
 0.60938,
   0.625,
 0.64063,
 0.65625,
 0.67188,
  0.6875,
 0.70313,
 0.71875,
 0.73438,
    0.75,
 0.76563,
 0.78125,
 0.79688,
  0.8125,
 0.82813,
 0.84375,
 0.85938,
   0.875,
 0.89063,
 0.90625,
 0.92188,
  0.9375,
 0.95313,
 0.96875,
 0.98438,
       1};
            #endregion
            public HotColormapLUT()
                : base(HotColormapLUT.R, HotColormapLUT.G, HotColormapLUT.B) { }
        }
        internal class JetColormapLUT : ColormapLUT
        {
            #region Colormap Definitions
            static double[] R = new double[]
        {
                   0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
0.015625,
 0.03125,
0.046875,
  0.0625,
0.078125,
 0.09375,
 0.10938,
   0.125,
 0.14063,
 0.15625,
 0.17188,
  0.1875,
 0.20313,
 0.21875,
 0.23438,
    0.25,
 0.26563,
 0.28125,
 0.29688,
  0.3125,
 0.32813,
 0.34375,
 0.35938,
   0.375,
 0.39063,
 0.40625,
 0.42188,
  0.4375,
 0.45313,
 0.46875,
 0.48438,
     0.5,
 0.51563,
 0.53125,
 0.54688,
  0.5625,
 0.57813,
 0.59375,
 0.60938,
   0.625,
 0.64063,
 0.65625,
 0.67188,
  0.6875,
 0.70313,
 0.71875,
 0.73438,
    0.75,
 0.76563,
 0.78125,
 0.79688,
  0.8125,
 0.82813,
 0.84375,
 0.85938,
   0.875,
 0.89063,
 0.90625,
 0.92188,
  0.9375,
 0.95313,
 0.96875,
 0.98438,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
 0.98438,
 0.96875,
 0.95313,
  0.9375,
 0.92188,
 0.90625,
 0.89063,
   0.875,
 0.85938,
 0.84375,
 0.82813,
  0.8125,
 0.79688,
 0.78125,
 0.76563,
    0.75,
 0.73438,
 0.71875,
 0.70313,
  0.6875,
 0.67188,
 0.65625,
 0.64063,
   0.625,
 0.60938,
 0.59375,
 0.57813,
  0.5625,
 0.54688,
 0.53125,
 0.51563,
     0.5,
        };

            static double[] G = new double[]{
           0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
0.015625,
 0.03125,
0.046875,
  0.0625,
0.078125,
 0.09375,
 0.10938,
   0.125,
 0.14063,
 0.15625,
 0.17188,
  0.1875,
 0.20313,
 0.21875,
 0.23438,
    0.25,
 0.26563,
 0.28125,
 0.29688,
  0.3125,
 0.32813,
 0.34375,
 0.35938,
   0.375,
 0.39063,
 0.40625,
 0.42188,
  0.4375,
 0.45313,
 0.46875,
 0.48438,
     0.5,
 0.51563,
 0.53125,
 0.54688,
  0.5625,
 0.57813,
 0.59375,
 0.60938,
   0.625,
 0.64063,
 0.65625,
 0.67188,
  0.6875,
 0.70313,
 0.71875,
 0.73438,
    0.75,
 0.76563,
 0.78125,
 0.79688,
  0.8125,
 0.82813,
 0.84375,
 0.85938,
   0.875,
 0.89063,
 0.90625,
 0.92188,
  0.9375,
 0.95313,
 0.96875,
 0.98438,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
 0.98438,
 0.96875,
 0.95313,
  0.9375,
 0.92188,
 0.90625,
 0.89063,
   0.875,
 0.85938,
 0.84375,
 0.82813,
  0.8125,
 0.79688,
 0.78125,
 0.76563,
    0.75,
 0.73438,
 0.71875,
 0.70313,
  0.6875,
 0.67188,
 0.65625,
 0.64063,
   0.625,
 0.60938,
 0.59375,
 0.57813,
  0.5625,
 0.54688,
 0.53125,
 0.51563,
     0.5,
 0.48438,
 0.46875,
 0.45313,
  0.4375,
 0.42188,
 0.40625,
 0.39063,
   0.375,
 0.35938,
 0.34375,
 0.32813,
  0.3125,
 0.29688,
 0.28125,
 0.26563,
    0.25,
 0.23438,
 0.21875,
 0.20313,
  0.1875,
 0.17188,
 0.15625,
 0.14063,
   0.125,
 0.10938,
 0.09375,
0.078125,
  0.0625,
0.046875,
 0.03125,
0.015625,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
};

            static double[] B = new double[]{     
 0.51563,
 0.53125,
 0.54688,
  0.5625,
 0.57813,
 0.59375,
 0.60938,
   0.625,
 0.64063,
 0.65625,
 0.67188,
  0.6875,
 0.70313,
 0.71875,
 0.73438,
    0.75,
 0.76563,
 0.78125,
 0.79688,
  0.8125,
 0.82813,
 0.84375,
 0.85938,
   0.875,
 0.89063,
 0.90625,
 0.92188,
  0.9375,
 0.95313,
 0.96875,
 0.98438,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
 0.98438,
 0.96875,
 0.95313,
  0.9375,
 0.92188,
 0.90625,
 0.89063,
   0.875,
 0.85938,
 0.84375,
 0.82813,
  0.8125,
 0.79688,
 0.78125,
 0.76563,
    0.75,
 0.73438,
 0.71875,
 0.70313,
  0.6875,
 0.67188,
 0.65625,
 0.64063,
   0.625,
 0.60938,
 0.59375,
 0.57813,
  0.5625,
 0.54688,
 0.53125,
 0.51563,
     0.5,
 0.48438,
 0.46875,
 0.45313,
  0.4375,
 0.42188,
 0.40625,
 0.39063,
   0.375,
 0.35938,
 0.34375,
 0.32813,
  0.3125,
 0.29688,
 0.28125,
 0.26563,
    0.25,
 0.23438,
 0.21875,
 0.20313,
  0.1875,
 0.17188,
 0.15625,
 0.14063,
   0.125,
 0.10938,
 0.09375,
0.078125,
  0.0625,
0.046875,
 0.03125,
0.015625,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
};

            #endregion
            internal JetColormapLUT()
                : base(JetColormapLUT.R, JetColormapLUT.G, JetColormapLUT.B) { }
        }
        internal class GrayColormapLUT : ColormapLUT
        {
            #region Colormap Definitions
            static double[] R = new double[]
        {
0,
0.0039216,
0.0078431,
 0.011765,
 0.015686,
 0.019608,
 0.023529,
 0.027451,
 0.031373,
 0.035294,
 0.039216,
 0.043137,
 0.047059,
  0.05098,
 0.054902,
 0.058824,
 0.062745,
 0.066667,
 0.070588,
  0.07451,
 0.078431,
 0.082353,
 0.086275,
 0.090196,
 0.094118,
 0.098039,
  0.10196,
  0.10588,
   0.1098,
  0.11373,
  0.11765,
  0.12157,
  0.12549,
  0.12941,
  0.13333,
  0.13725,
  0.14118,
   0.1451,
  0.14902,
  0.15294,
  0.15686,
  0.16078,
  0.16471,
  0.16863,
  0.17255,
  0.17647,
  0.18039,
  0.18431,
  0.18824,
  0.19216,
  0.19608,
      0.2,
  0.20392,
  0.20784,
  0.21176,
  0.21569,
  0.21961,
  0.22353,
  0.22745,
  0.23137,
  0.23529,
  0.23922,
  0.24314,
  0.24706,
  0.25098,
   0.2549,
  0.25882,
  0.26275,
  0.26667,
  0.27059,
  0.27451,
  0.27843,
  0.28235,
  0.28627,
   0.2902,
  0.29412,
  0.29804,
  0.30196,
  0.30588,
   0.3098,
  0.31373,
  0.31765,
  0.32157,
  0.32549,
  0.32941,
  0.33333,
  0.33725,
  0.34118,
   0.3451,
  0.34902,
  0.35294,
  0.35686,
  0.36078,
  0.36471,
  0.36863,
  0.37255,
  0.37647,
  0.38039,
  0.38431,
  0.38824,
  0.39216,
  0.39608,
      0.4,
  0.40392,
  0.40784,
  0.41176,
  0.41569,
  0.41961,
  0.42353,
  0.42745,
  0.43137,
  0.43529,
  0.43922,
  0.44314,
  0.44706,
  0.45098,
   0.4549,
  0.45882,
  0.46275,
  0.46667,
  0.47059,
  0.47451,
  0.47843,
  0.48235,
  0.48627,
   0.4902,
  0.49412,
  0.49804,
  0.50196,
  0.50588,
   0.5098,
  0.51373,
  0.51765,
  0.52157,
  0.52549,
  0.52941,
  0.53333,
  0.53725,
  0.54118,
   0.5451,
  0.54902,
  0.55294,
  0.55686,
  0.56078,
  0.56471,
  0.56863,
  0.57255,
  0.57647,
  0.58039,
  0.58431,
  0.58824,
  0.59216,
  0.59608,
      0.6,
  0.60392,
  0.60784,
  0.61176,
  0.61569,
  0.61961,
  0.62353,
  0.62745,
  0.63137,
  0.63529,
  0.63922,
  0.64314,
  0.64706,
  0.65098,
   0.6549,
  0.65882,
  0.66275,
  0.66667,
  0.67059,
  0.67451,
  0.67843,
  0.68235,
  0.68627,
   0.6902,
  0.69412,
  0.69804,
  0.70196,
  0.70588,
   0.7098,
  0.71373,
  0.71765,
  0.72157,
  0.72549,
  0.72941,
  0.73333,
  0.73725,
  0.74118,
   0.7451,
  0.74902,
  0.75294,
  0.75686,
  0.76078,
  0.76471,
  0.76863,
  0.77255,
  0.77647,
  0.78039,
  0.78431,
  0.78824,
  0.79216,
  0.79608,
      0.8,
  0.80392,
  0.80784,
  0.81176,
  0.81569,
  0.81961,
  0.82353,
  0.82745,
  0.83137,
  0.83529,
  0.83922,
  0.84314,
  0.84706,
  0.85098,
   0.8549,
  0.85882,
  0.86275,
  0.86667,
  0.87059,
  0.87451,
  0.87843,
  0.88235,
  0.88627,
   0.8902,
  0.89412,
  0.89804,
  0.90196,
  0.90588,
   0.9098,
  0.91373,
  0.91765,
  0.92157,
  0.92549,
  0.92941,
  0.93333,
  0.93725,
  0.94118,
   0.9451,
  0.94902,
  0.95294,
  0.95686,
  0.96078,
  0.96471,
  0.96863,
  0.97255,
  0.97647,
  0.98039,
  0.98431,
  0.98824,
  0.99216,
  0.99608,
        1,
        };
            static double[] G = new double[]
        {
0,
0.0039216,
0.0078431,
 0.011765,
 0.015686,
 0.019608,
 0.023529,
 0.027451,
 0.031373,
 0.035294,
 0.039216,
 0.043137,
 0.047059,
  0.05098,
 0.054902,
 0.058824,
 0.062745,
 0.066667,
 0.070588,
  0.07451,
 0.078431,
 0.082353,
 0.086275,
 0.090196,
 0.094118,
 0.098039,
  0.10196,
  0.10588,
   0.1098,
  0.11373,
  0.11765,
  0.12157,
  0.12549,
  0.12941,
  0.13333,
  0.13725,
  0.14118,
   0.1451,
  0.14902,
  0.15294,
  0.15686,
  0.16078,
  0.16471,
  0.16863,
  0.17255,
  0.17647,
  0.18039,
  0.18431,
  0.18824,
  0.19216,
  0.19608,
      0.2,
  0.20392,
  0.20784,
  0.21176,
  0.21569,
  0.21961,
  0.22353,
  0.22745,
  0.23137,
  0.23529,
  0.23922,
  0.24314,
  0.24706,
  0.25098,
   0.2549,
  0.25882,
  0.26275,
  0.26667,
  0.27059,
  0.27451,
  0.27843,
  0.28235,
  0.28627,
   0.2902,
  0.29412,
  0.29804,
  0.30196,
  0.30588,
   0.3098,
  0.31373,
  0.31765,
  0.32157,
  0.32549,
  0.32941,
  0.33333,
  0.33725,
  0.34118,
   0.3451,
  0.34902,
  0.35294,
  0.35686,
  0.36078,
  0.36471,
  0.36863,
  0.37255,
  0.37647,
  0.38039,
  0.38431,
  0.38824,
  0.39216,
  0.39608,
      0.4,
  0.40392,
  0.40784,
  0.41176,
  0.41569,
  0.41961,
  0.42353,
  0.42745,
  0.43137,
  0.43529,
  0.43922,
  0.44314,
  0.44706,
  0.45098,
   0.4549,
  0.45882,
  0.46275,
  0.46667,
  0.47059,
  0.47451,
  0.47843,
  0.48235,
  0.48627,
   0.4902,
  0.49412,
  0.49804,
  0.50196,
  0.50588,
   0.5098,
  0.51373,
  0.51765,
  0.52157,
  0.52549,
  0.52941,
  0.53333,
  0.53725,
  0.54118,
   0.5451,
  0.54902,
  0.55294,
  0.55686,
  0.56078,
  0.56471,
  0.56863,
  0.57255,
  0.57647,
  0.58039,
  0.58431,
  0.58824,
  0.59216,
  0.59608,
      0.6,
  0.60392,
  0.60784,
  0.61176,
  0.61569,
  0.61961,
  0.62353,
  0.62745,
  0.63137,
  0.63529,
  0.63922,
  0.64314,
  0.64706,
  0.65098,
   0.6549,
  0.65882,
  0.66275,
  0.66667,
  0.67059,
  0.67451,
  0.67843,
  0.68235,
  0.68627,
   0.6902,
  0.69412,
  0.69804,
  0.70196,
  0.70588,
   0.7098,
  0.71373,
  0.71765,
  0.72157,
  0.72549,
  0.72941,
  0.73333,
  0.73725,
  0.74118,
   0.7451,
  0.74902,
  0.75294,
  0.75686,
  0.76078,
  0.76471,
  0.76863,
  0.77255,
  0.77647,
  0.78039,
  0.78431,
  0.78824,
  0.79216,
  0.79608,
      0.8,
  0.80392,
  0.80784,
  0.81176,
  0.81569,
  0.81961,
  0.82353,
  0.82745,
  0.83137,
  0.83529,
  0.83922,
  0.84314,
  0.84706,
  0.85098,
   0.8549,
  0.85882,
  0.86275,
  0.86667,
  0.87059,
  0.87451,
  0.87843,
  0.88235,
  0.88627,
   0.8902,
  0.89412,
  0.89804,
  0.90196,
  0.90588,
   0.9098,
  0.91373,
  0.91765,
  0.92157,
  0.92549,
  0.92941,
  0.93333,
  0.93725,
  0.94118,
   0.9451,
  0.94902,
  0.95294,
  0.95686,
  0.96078,
  0.96471,
  0.96863,
  0.97255,
  0.97647,
  0.98039,
  0.98431,
  0.98824,
  0.99216,
  0.99608,
        1,
        };
            static double[] B = new double[]
        {
0,
0.0039216,
0.0078431,
 0.011765,
 0.015686,
 0.019608,
 0.023529,
 0.027451,
 0.031373,
 0.035294,
 0.039216,
 0.043137,
 0.047059,
  0.05098,
 0.054902,
 0.058824,
 0.062745,
 0.066667,
 0.070588,
  0.07451,
 0.078431,
 0.082353,
 0.086275,
 0.090196,
 0.094118,
 0.098039,
  0.10196,
  0.10588,
   0.1098,
  0.11373,
  0.11765,
  0.12157,
  0.12549,
  0.12941,
  0.13333,
  0.13725,
  0.14118,
   0.1451,
  0.14902,
  0.15294,
  0.15686,
  0.16078,
  0.16471,
  0.16863,
  0.17255,
  0.17647,
  0.18039,
  0.18431,
  0.18824,
  0.19216,
  0.19608,
      0.2,
  0.20392,
  0.20784,
  0.21176,
  0.21569,
  0.21961,
  0.22353,
  0.22745,
  0.23137,
  0.23529,
  0.23922,
  0.24314,
  0.24706,
  0.25098,
   0.2549,
  0.25882,
  0.26275,
  0.26667,
  0.27059,
  0.27451,
  0.27843,
  0.28235,
  0.28627,
   0.2902,
  0.29412,
  0.29804,
  0.30196,
  0.30588,
   0.3098,
  0.31373,
  0.31765,
  0.32157,
  0.32549,
  0.32941,
  0.33333,
  0.33725,
  0.34118,
   0.3451,
  0.34902,
  0.35294,
  0.35686,
  0.36078,
  0.36471,
  0.36863,
  0.37255,
  0.37647,
  0.38039,
  0.38431,
  0.38824,
  0.39216,
  0.39608,
      0.4,
  0.40392,
  0.40784,
  0.41176,
  0.41569,
  0.41961,
  0.42353,
  0.42745,
  0.43137,
  0.43529,
  0.43922,
  0.44314,
  0.44706,
  0.45098,
   0.4549,
  0.45882,
  0.46275,
  0.46667,
  0.47059,
  0.47451,
  0.47843,
  0.48235,
  0.48627,
   0.4902,
  0.49412,
  0.49804,
  0.50196,
  0.50588,
   0.5098,
  0.51373,
  0.51765,
  0.52157,
  0.52549,
  0.52941,
  0.53333,
  0.53725,
  0.54118,
   0.5451,
  0.54902,
  0.55294,
  0.55686,
  0.56078,
  0.56471,
  0.56863,
  0.57255,
  0.57647,
  0.58039,
  0.58431,
  0.58824,
  0.59216,
  0.59608,
      0.6,
  0.60392,
  0.60784,
  0.61176,
  0.61569,
  0.61961,
  0.62353,
  0.62745,
  0.63137,
  0.63529,
  0.63922,
  0.64314,
  0.64706,
  0.65098,
   0.6549,
  0.65882,
  0.66275,
  0.66667,
  0.67059,
  0.67451,
  0.67843,
  0.68235,
  0.68627,
   0.6902,
  0.69412,
  0.69804,
  0.70196,
  0.70588,
   0.7098,
  0.71373,
  0.71765,
  0.72157,
  0.72549,
  0.72941,
  0.73333,
  0.73725,
  0.74118,
   0.7451,
  0.74902,
  0.75294,
  0.75686,
  0.76078,
  0.76471,
  0.76863,
  0.77255,
  0.77647,
  0.78039,
  0.78431,
  0.78824,
  0.79216,
  0.79608,
      0.8,
  0.80392,
  0.80784,
  0.81176,
  0.81569,
  0.81961,
  0.82353,
  0.82745,
  0.83137,
  0.83529,
  0.83922,
  0.84314,
  0.84706,
  0.85098,
   0.8549,
  0.85882,
  0.86275,
  0.86667,
  0.87059,
  0.87451,
  0.87843,
  0.88235,
  0.88627,
   0.8902,
  0.89412,
  0.89804,
  0.90196,
  0.90588,
   0.9098,
  0.91373,
  0.91765,
  0.92157,
  0.92549,
  0.92941,
  0.93333,
  0.93725,
  0.94118,
   0.9451,
  0.94902,
  0.95294,
  0.95686,
  0.96078,
  0.96471,
  0.96863,
  0.97255,
  0.97647,
  0.98039,
  0.98431,
  0.98824,
  0.99216,
  0.99608,
        1,
        };
            #endregion
            internal GrayColormapLUT()
                : base(GrayColormapLUT.R, GrayColormapLUT.G, GrayColormapLUT.B) { }
        }
        internal class BoneColormapLUT : ColormapLUT
        {
            #region Colormap Definitions
            static double[] R = new double[]{
        0,
0.0034314,
0.0068627,
 0.010294,
 0.013725,
 0.017157,
 0.020588,
  0.02402,
 0.027451,
 0.030882,
 0.034314,
 0.037745,
 0.041176,
 0.044608,
 0.048039,
 0.051471,
 0.054902,
 0.058333,
 0.061765,
 0.065196,
 0.068627,
 0.072059,
  0.07549,
 0.078922,
 0.082353,
 0.085784,
 0.089216,
 0.092647,
 0.096078,
  0.09951,
  0.10294,
  0.10637,
   0.1098,
  0.11324,
  0.11667,
   0.1201,
  0.12353,
  0.12696,
  0.13039,
  0.13382,
  0.13725,
  0.14069,
  0.14412,
  0.14755,
  0.15098,
  0.15441,
  0.15784,
  0.16127,
  0.16471,
  0.16814,
  0.17157,
    0.175,
  0.17843,
  0.18186,
  0.18529,
  0.18873,
  0.19216,
  0.19559,
  0.19902,
  0.20245,
  0.20588,
  0.20931,
  0.21275,
  0.21618,
  0.21961,
  0.22304,
  0.22647,
   0.2299,
  0.23333,
  0.23676,
   0.2402,
  0.24363,
  0.24706,
  0.25049,
  0.25392,
  0.25735,
  0.26078,
  0.26422,
  0.26765,
  0.27108,
  0.27451,
  0.27794,
  0.28137,
   0.2848,
  0.28824,
  0.29167,
   0.2951,
  0.29853,
  0.30196,
  0.30539,
  0.30882,
  0.31225,
  0.31569,
  0.31912,
  0.32255,
  0.32598,
  0.32941,
  0.33284,
  0.33627,
  0.33971,
  0.34314,
  0.34657,
     0.35,
  0.35343,
  0.35686,
  0.36029,
  0.36373,
  0.36716,
  0.37059,
  0.37402,
  0.37745,
  0.38088,
  0.38431,
  0.38775,
  0.39118,
  0.39461,
  0.39804,
  0.40147,
   0.4049,
  0.40833,
  0.41176,
   0.4152,
  0.41863,
  0.42206,
  0.42549,
  0.42892,
  0.43235,
  0.43578,
  0.43922,
  0.44265,
  0.44608,
  0.44951,
  0.45294,
  0.45637,
   0.4598,
  0.46324,
  0.46667,
   0.4701,
  0.47353,
  0.47696,
  0.48039,
  0.48382,
  0.48725,
  0.49069,
  0.49412,
  0.49755,
  0.50098,
  0.50441,
  0.50784,
  0.51127,
  0.51471,
  0.51814,
  0.52157,
    0.525,
  0.52843,
  0.53186,
  0.53529,
  0.53873,
  0.54216,
  0.54559,
  0.54902,
  0.55245,
  0.55588,
  0.55931,
  0.56275,
  0.56618,
  0.56961,
  0.57304,
  0.57647,
   0.5799,
  0.58333,
  0.58676,
   0.5902,
  0.59363,
  0.59706,
  0.60049,
  0.60392,
  0.60735,
  0.61078,
  0.61422,
  0.61765,
  0.62108,
  0.62451,
  0.62794,
  0.63137,
   0.6348,
  0.63824,
  0.64167,
   0.6451,
  0.64853,
  0.65196,
  0.65539,
  0.66078,
  0.66616,
  0.67155,
  0.67693,
  0.68231,
   0.6877,
  0.69308,
  0.69847,
  0.70385,
  0.70924,
  0.71462,
  0.72001,
  0.72539,
  0.73078,
  0.73616,
  0.74154,
  0.74693,
  0.75231,
   0.7577,
  0.76308,
  0.76847,
  0.77385,
  0.77924,
  0.78462,
     0.79,
  0.79539,
  0.80077,
  0.80616,
  0.81154,
  0.81693,
  0.82231,
   0.8277,
  0.83308,
  0.83847,
  0.84385,
  0.84923,
  0.85462,
     0.86,
  0.86539,
  0.87077,
  0.87616,
  0.88154,
  0.88693,
  0.89231,
  0.89769,
  0.90308,
  0.90846,
  0.91385,
  0.91923,
  0.92462,
     0.93,
  0.93539,
  0.94077,
  0.94616,
  0.95154,
  0.95692,
  0.96231,
  0.96769,
  0.97308,
  0.97846,
  0.98385,
  0.98923,
  0.99462,
        1,
};
            static double[] G = new double[]{
        0,
0.0034314,
0.0068627,
 0.010294,
 0.013725,
 0.017157,
 0.020588,
  0.02402,
 0.027451,
 0.030882,
 0.034314,
 0.037745,
 0.041176,
 0.044608,
 0.048039,
 0.051471,
 0.054902,
 0.058333,
 0.061765,
 0.065196,
 0.068627,
 0.072059,
  0.07549,
 0.078922,
 0.082353,
 0.085784,
 0.089216,
 0.092647,
 0.096078,
  0.09951,
  0.10294,
  0.10637,
   0.1098,
  0.11324,
  0.11667,
   0.1201,
  0.12353,
  0.12696,
  0.13039,
  0.13382,
  0.13725,
  0.14069,
  0.14412,
  0.14755,
  0.15098,
  0.15441,
  0.15784,
  0.16127,
  0.16471,
  0.16814,
  0.17157,
    0.175,
  0.17843,
  0.18186,
  0.18529,
  0.18873,
  0.19216,
  0.19559,
  0.19902,
  0.20245,
  0.20588,
  0.20931,
  0.21275,
  0.21618,
  0.21961,
  0.22304,
  0.22647,
   0.2299,
  0.23333,
  0.23676,
   0.2402,
  0.24363,
  0.24706,
  0.25049,
  0.25392,
  0.25735,
  0.26078,
  0.26422,
  0.26765,
  0.27108,
  0.27451,
  0.27794,
  0.28137,
   0.2848,
  0.28824,
  0.29167,
   0.2951,
  0.29853,
  0.30196,
  0.30539,
  0.30882,
  0.31225,
  0.31569,
  0.31912,
  0.32255,
  0.32598,
  0.33071,
  0.33545,
  0.34018,
  0.34491,
  0.34965,
  0.35438,
  0.35911,
  0.36385,
  0.36858,
  0.37331,
  0.37805,
  0.38278,
  0.38752,
  0.39225,
  0.39698,
  0.40172,
  0.40645,
  0.41118,
  0.41592,
  0.42065,
  0.42538,
  0.43012,
  0.43485,
  0.43958,
  0.44432,
  0.44905,
  0.45378,
  0.45852,
  0.46325,
  0.46798,
  0.47272,
  0.47745,
  0.48218,
  0.48692,
  0.49165,
  0.49638,
  0.50112,
  0.50585,
  0.51059,
  0.51532,
  0.52005,
  0.52479,
  0.52952,
  0.53425,
  0.53899,
  0.54372,
  0.54845,
  0.55319,
  0.55792,
  0.56265,
  0.56739,
  0.57212,
  0.57685,
  0.58159,
  0.58632,
  0.59105,
  0.59579,
  0.60052,
  0.60525,
  0.60999,
  0.61472,
  0.61945,
  0.62419,
  0.62892,
  0.63366,
  0.63839,
  0.64312,
  0.64786,
  0.65259,
  0.65732,
  0.66206,
  0.66679,
  0.67152,
  0.67626,
  0.68099,
  0.68572,
  0.69046,
  0.69519,
  0.69992,
  0.70466,
  0.70939,
  0.71412,
  0.71886,
  0.72359,
  0.72832,
  0.73306,
  0.73779,
  0.74252,
  0.74726,
  0.75199,
  0.75672,
  0.76146,
  0.76619,
  0.77093,
  0.77566,
  0.78039,
  0.78382,
  0.78725,
  0.79069,
  0.79412,
  0.79755,
  0.80098,
  0.80441,
  0.80784,
  0.81127,
  0.81471,
  0.81814,
  0.82157,
    0.825,
  0.82843,
  0.83186,
  0.83529,
  0.83873,
  0.84216,
  0.84559,
  0.84902,
  0.85245,
  0.85588,
  0.85931,
  0.86275,
  0.86618,
  0.86961,
  0.87304,
  0.87647,
   0.8799,
  0.88333,
  0.88676,
   0.8902,
  0.89363,
  0.89706,
  0.90049,
  0.90392,
  0.90735,
  0.91078,
  0.91422,
  0.91765,
  0.92108,
  0.92451,
  0.92794,
  0.93137,
   0.9348,
  0.93824,
  0.94167,
   0.9451,
  0.94853,
  0.95196,
  0.95539,
  0.95882,
  0.96225,
  0.96569,
  0.96912,
  0.97255,
  0.97598,
  0.97941,
  0.98284,
  0.98627,
  0.98971,
  0.99314,
  0.99657,
        1,
};
            static double[] B = new double[]{     
0.0013021,
0.0060355,
 0.010769,
 0.015502,
 0.020236,
 0.024969,
 0.029703,
 0.034436,
  0.03917,
 0.043903,
 0.048637,
  0.05337,
 0.058104,
 0.062837,
  0.06757,
 0.072304,
 0.077037,
 0.081771,
 0.086504,
 0.091238,
 0.095971,
   0.1007,
  0.10544,
  0.11017,
  0.11491,
  0.11964,
  0.12437,
  0.12911,
  0.13384,
  0.13857,
  0.14331,
  0.14804,
  0.15277,
  0.15751,
  0.16224,
  0.16697,
  0.17171,
  0.17644,
  0.18117,
  0.18591,
  0.19064,
  0.19537,
  0.20011,
  0.20484,
  0.20957,
  0.21431,
  0.21904,
  0.22377,
  0.22851,
  0.23324,
  0.23797,
  0.24271,
  0.24744,
  0.25218,
  0.25691,
  0.26164,
  0.26638,
  0.27111,
  0.27584,
  0.28058,
  0.28531,
  0.29004,
  0.29478,
  0.29951,
  0.30424,
  0.30898,
  0.31371,
  0.31844,
  0.32318,
  0.32791,
  0.33264,
  0.33738,
  0.34211,
  0.34684,
  0.35158,
  0.35631,
  0.36104,
  0.36578,
  0.37051,
  0.37525,
  0.37998,
  0.38471,
  0.38945,
  0.39418,
  0.39891,
  0.40365,
  0.40838,
  0.41311,
  0.41785,
  0.42258,
  0.42731,
  0.43205,
  0.43678,
  0.44151,
  0.44625,
  0.45098,
  0.45441,
  0.45784,
  0.46127,
  0.46471,
  0.46814,
  0.47157,
    0.475,
  0.47843,
  0.48186,
  0.48529,
  0.48873,
  0.49216,
  0.49559,
  0.49902,
  0.50245,
  0.50588,
  0.50931,
  0.51275,
  0.51618,
  0.51961,
  0.52304,
  0.52647,
   0.5299,
  0.53333,
  0.53676,
   0.5402,
  0.54363,
  0.54706,
  0.55049,
  0.55392,
  0.55735,
  0.56078,
  0.56422,
  0.56765,
  0.57108,
  0.57451,
  0.57794,
  0.58137,
   0.5848,
  0.58824,
  0.59167,
   0.5951,
  0.59853,
  0.60196,
  0.60539,
  0.60882,
  0.61225,
  0.61569,
  0.61912,
  0.62255,
  0.62598,
  0.62941,
  0.63284,
  0.63627,
  0.63971,
  0.64314,
  0.64657,
     0.65,
  0.65343,
  0.65686,
  0.66029,
  0.66373,
  0.66716,
  0.67059,
  0.67402,
  0.67745,
  0.68088,
  0.68431,
  0.68775,
  0.69118,
  0.69461,
  0.69804,
  0.70147,
   0.7049,
  0.70833,
  0.71176,
   0.7152,
  0.71863,
  0.72206,
  0.72549,
  0.72892,
  0.73235,
  0.73578,
  0.73922,
  0.74265,
  0.74608,
  0.74951,
  0.75294,
  0.75637,
   0.7598,
  0.76324,
  0.76667,
   0.7701,
  0.77353,
  0.77696,
  0.78039,
  0.78382,
  0.78725,
  0.79069,
  0.79412,
  0.79755,
  0.80098,
  0.80441,
  0.80784,
  0.81127,
  0.81471,
  0.81814,
  0.82157,
    0.825,
  0.82843,
  0.83186,
  0.83529,
  0.83873,
  0.84216,
  0.84559,
  0.84902,
  0.85245,
  0.85588,
  0.85931,
  0.86275,
  0.86618,
  0.86961,
  0.87304,
  0.87647,
   0.8799,
  0.88333,
  0.88676,
   0.8902,
  0.89363,
  0.89706,
  0.90049,
  0.90392,
  0.90735,
  0.91078,
  0.91422,
  0.91765,
  0.92108,
  0.92451,
  0.92794,
  0.93137,
   0.9348,
  0.93824,
  0.94167,
   0.9451,
  0.94853,
  0.95196,
  0.95539,
  0.95882,
  0.96225,
  0.96569,
  0.96912,
  0.97255,
  0.97598,
  0.97941,
  0.98284,
  0.98627,
  0.98971,
  0.99314,
  0.99657,
        1,
};
            #endregion
            internal BoneColormapLUT()
                : base(BoneColormapLUT.R, BoneColormapLUT.G, BoneColormapLUT.B) { }
        }
        internal class CopperColormapLUT : ColormapLUT
        {
            #region Colormap Definitions
            static double[] R = new double[]{
        0,
 0.004902,
0.0098039,
 0.014706,
 0.019608,
  0.02451,
 0.029412,
 0.034314,
 0.039216,
 0.044118,
  0.04902,
 0.053922,
 0.058824,
 0.063725,
 0.068627,
 0.073529,
 0.078431,
 0.083333,
 0.088235,
 0.093137,
 0.098039,
  0.10294,
  0.10784,
  0.11275,
  0.11765,
  0.12255,
  0.12745,
  0.13235,
  0.13725,
  0.14216,
  0.14706,
  0.15196,
  0.15686,
  0.16176,
  0.16667,
  0.17157,
  0.17647,
  0.18137,
  0.18627,
  0.19118,
  0.19608,
  0.20098,
  0.20588,
  0.21078,
  0.21569,
  0.22059,
  0.22549,
  0.23039,
  0.23529,
   0.2402,
   0.2451,
     0.25,
   0.2549,
   0.2598,
  0.26471,
  0.26961,
  0.27451,
  0.27941,
  0.28431,
  0.28922,
  0.29412,
  0.29902,
  0.30392,
  0.30882,
  0.31373,
  0.31863,
  0.32353,
  0.32843,
  0.33333,
  0.33824,
  0.34314,
  0.34804,
  0.35294,
  0.35784,
  0.36275,
  0.36765,
  0.37255,
  0.37745,
  0.38235,
  0.38725,
  0.39216,
  0.39706,
  0.40196,
  0.40686,
  0.41176,
  0.41667,
  0.42157,
  0.42647,
  0.43137,
  0.43627,
  0.44118,
  0.44608,
  0.45098,
  0.45588,
  0.46078,
  0.46569,
  0.47059,
  0.47549,
  0.48039,
  0.48529,
   0.4902,
   0.4951,
      0.5,
   0.5049,
   0.5098,
  0.51471,
  0.51961,
  0.52451,
  0.52941,
  0.53431,
  0.53922,
  0.54412,
  0.54902,
  0.55392,
  0.55882,
  0.56373,
  0.56863,
  0.57353,
  0.57843,
  0.58333,
  0.58824,
  0.59314,
  0.59804,
  0.60294,
  0.60784,
  0.61275,
  0.61765,
  0.62255,
  0.62745,
  0.63235,
  0.63725,
  0.64216,
  0.64706,
  0.65196,
  0.65686,
  0.66176,
  0.66667,
  0.67157,
  0.67647,
  0.68137,
  0.68627,
  0.69118,
  0.69608,
  0.70098,
  0.70588,
  0.71078,
  0.71569,
  0.72059,
  0.72549,
  0.73039,
  0.73529,
   0.7402,
   0.7451,
     0.75,
   0.7549,
   0.7598,
  0.76471,
  0.76961,
  0.77451,
  0.77941,
  0.78431,
  0.78922,
  0.79412,
  0.79902,
  0.80392,
  0.80882,
  0.81373,
  0.81863,
  0.82353,
  0.82843,
  0.83333,
  0.83824,
  0.84314,
  0.84804,
  0.85294,
  0.85784,
  0.86275,
  0.86765,
  0.87255,
  0.87745,
  0.88235,
  0.88725,
  0.89216,
  0.89706,
  0.90196,
  0.90686,
  0.91176,
  0.91667,
  0.92157,
  0.92647,
  0.93137,
  0.93627,
  0.94118,
  0.94608,
  0.95098,
  0.95588,
  0.96078,
  0.96569,
  0.97059,
  0.97549,
  0.98039,
  0.98529,
   0.9902,
   0.9951,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        };

            static double[] G = new double[]{
        0,
0.0030635,
0.0061271,
0.0091906,
 0.012254,
 0.015318,
 0.018381,
 0.021445,
 0.024508,
 0.027572,
 0.030635,
 0.033699,
 0.036762,
 0.039826,
 0.042889,
 0.045953,
 0.049016,
  0.05208,
 0.055144,
 0.058207,
 0.061271,
 0.064334,
 0.067398,
 0.070461,
 0.073525,
 0.076588,
 0.079652,
 0.082715,
 0.085779,
 0.088842,
 0.091906,
 0.094969,
 0.098033,
   0.1011,
  0.10416,
  0.10722,
  0.11029,
  0.11335,
  0.11641,
  0.11948,
  0.12254,
   0.1256,
  0.12867,
  0.13173,
   0.1348,
  0.13786,
  0.14092,
  0.14399,
  0.14705,
  0.15011,
  0.15318,
  0.15624,
   0.1593,
  0.16237,
  0.16543,
  0.16849,
  0.17156,
  0.17462,
  0.17768,
  0.18075,
  0.18381,
  0.18688,
  0.18994,
    0.193,
  0.19607,
  0.19913,
  0.20219,
  0.20526,
  0.20832,
  0.21138,
  0.21445,
  0.21751,
  0.22057,
  0.22364,
   0.2267,
  0.22976,
  0.23283,
  0.23589,
  0.23896,
  0.24202,
  0.24508,
  0.24815,
  0.25121,
  0.25427,
  0.25734,
   0.2604,
  0.26346,
  0.26653,
  0.26959,
  0.27265,
  0.27572,
  0.27878,
  0.28184,
  0.28491,
  0.28797,
  0.29104,
   0.2941,
  0.29716,
  0.30023,
  0.30329,
  0.30635,
  0.30942,
  0.31248,
  0.31554,
  0.31861,
  0.32167,
  0.32473,
   0.3278,
  0.33086,
  0.33392,
  0.33699,
  0.34005,
  0.34312,
  0.34618,
  0.34924,
  0.35231,
  0.35537,
  0.35843,
   0.3615,
  0.36456,
  0.36762,
  0.37069,
  0.37375,
  0.37681,
  0.37988,
  0.38294,
    0.386,
  0.38907,
  0.39213,
   0.3952,
  0.39826,
  0.40132,
  0.40439,
  0.40745,
  0.41051,
  0.41358,
  0.41664,
   0.4197,
  0.42277,
  0.42583,
  0.42889,
  0.43196,
  0.43502,
  0.43808,
  0.44115,
  0.44421,
  0.44728,
  0.45034,
   0.4534,
  0.45647,
  0.45953,
  0.46259,
  0.46566,
  0.46872,
  0.47178,
  0.47485,
  0.47791,
  0.48097,
  0.48404,
   0.4871,
  0.49016,
  0.49323,
  0.49629,
  0.49936,
  0.50242,
  0.50548,
  0.50855,
  0.51161,
  0.51467,
  0.51774,
   0.5208,
  0.52386,
  0.52693,
  0.52999,
  0.53305,
  0.53612,
  0.53918,
  0.54224,
  0.54531,
  0.54837,
  0.55144,
   0.5545,
  0.55756,
  0.56063,
  0.56369,
  0.56675,
  0.56982,
  0.57288,
  0.57594,
  0.57901,
  0.58207,
  0.58513,
   0.5882,
  0.59126,
  0.59432,
  0.59739,
  0.60045,
  0.60352,
  0.60658,
  0.60964,
  0.61271,
  0.61577,
  0.61883,
   0.6219,
  0.62496,
  0.62802,
  0.63109,
  0.63415,
  0.63721,
  0.64028,
  0.64334,
   0.6464,
  0.64947,
  0.65253,
   0.6556,
  0.65866,
  0.66172,
  0.66479,
  0.66785,
  0.67091,
  0.67398,
  0.67704,
   0.6801,
  0.68317,
  0.68623,
  0.68929,
  0.69236,
  0.69542,
  0.69848,
  0.70155,
  0.70461,
  0.70768,
  0.71074,
   0.7138,
  0.71687,
  0.71993,
  0.72299,
  0.72606,
  0.72912,
  0.73218,
  0.73525,
  0.73831,
  0.74137,
  0.74444,
   0.7475,
  0.75056,
  0.75363,
  0.75669,
  0.75976,
  0.76282,
  0.76588,
  0.76895,
  0.77201,
  0.77507,
  0.77814,
   0.7812,
};

            static double[] B = new double[]{     
        0,
 0.001951,
 0.003902,
0.0058529,
0.0078039,
0.0097549,
 0.011706,
 0.013657,
 0.015608,
 0.017559,
  0.01951,
 0.021461,
 0.023412,
 0.025363,
 0.027314,
 0.029265,
 0.031216,
 0.033167,
 0.035118,
 0.037069,
  0.03902,
 0.040971,
 0.042922,
 0.044873,
 0.046824,
 0.048775,
 0.050725,
 0.052676,
 0.054627,
 0.056578,
 0.058529,
  0.06048,
 0.062431,
 0.064382,
 0.066333,
 0.068284,
 0.070235,
 0.072186,
 0.074137,
 0.076088,
 0.078039,
  0.07999,
 0.081941,
 0.083892,
 0.085843,
 0.087794,
 0.089745,
 0.091696,
 0.093647,
 0.095598,
 0.097549,
   0.0995,
  0.10145,
   0.1034,
  0.10535,
   0.1073,
  0.10925,
  0.11121,
  0.11316,
  0.11511,
  0.11706,
  0.11901,
  0.12096,
  0.12291,
  0.12486,
  0.12681,
  0.12876,
  0.13072,
  0.13267,
  0.13462,
  0.13657,
  0.13852,
  0.14047,
  0.14242,
  0.14437,
  0.14632,
  0.14827,
  0.15023,
  0.15218,
  0.15413,
  0.15608,
  0.15803,
  0.15998,
  0.16193,
  0.16388,
  0.16583,
  0.16778,
  0.16974,
  0.17169,
  0.17364,
  0.17559,
  0.17754,
  0.17949,
  0.18144,
  0.18339,
  0.18534,
  0.18729,
  0.18925,
   0.1912,
  0.19315,
   0.1951,
  0.19705,
    0.199,
  0.20095,
   0.2029,
  0.20485,
   0.2068,
  0.20875,
  0.21071,
  0.21266,
  0.21461,
  0.21656,
  0.21851,
  0.22046,
  0.22241,
  0.22436,
  0.22631,
  0.22826,
  0.23022,
  0.23217,
  0.23412,
  0.23607,
  0.23802,
  0.23997,
  0.24192,
  0.24387,
  0.24582,
  0.24777,
  0.24973,
  0.25168,
  0.25363,
  0.25558,
  0.25753,
  0.25948,
  0.26143,
  0.26338,
  0.26533,
  0.26728,
  0.26924,
  0.27119,
  0.27314,
  0.27509,
  0.27704,
  0.27899,
  0.28094,
  0.28289,
  0.28484,
  0.28679,
  0.28875,
   0.2907,
  0.29265,
   0.2946,
  0.29655,
   0.2985,
  0.30045,
   0.3024,
  0.30435,
   0.3063,
  0.30825,
  0.31021,
  0.31216,
  0.31411,
  0.31606,
  0.31801,
  0.31996,
  0.32191,
  0.32386,
  0.32581,
  0.32776,
  0.32972,
  0.33167,
  0.33362,
  0.33557,
  0.33752,
  0.33947,
  0.34142,
  0.34337,
  0.34532,
  0.34727,
  0.34923,
  0.35118,
  0.35313,
  0.35508,
  0.35703,
  0.35898,
  0.36093,
  0.36288,
  0.36483,
  0.36678,
  0.36874,
  0.37069,
  0.37264,
  0.37459,
  0.37654,
  0.37849,
  0.38044,
  0.38239,
  0.38434,
  0.38629,
  0.38825,
   0.3902,
  0.39215,
   0.3941,
  0.39605,
    0.398,
  0.39995,
   0.4019,
  0.40385,
   0.4058,
  0.40775,
  0.40971,
  0.41166,
  0.41361,
  0.41556,
  0.41751,
  0.41946,
  0.42141,
  0.42336,
  0.42531,
  0.42726,
  0.42922,
  0.43117,
  0.43312,
  0.43507,
  0.43702,
  0.43897,
  0.44092,
  0.44287,
  0.44482,
  0.44677,
  0.44873,
  0.45068,
  0.45263,
  0.45458,
  0.45653,
  0.45848,
  0.46043,
  0.46238,
  0.46433,
  0.46628,
  0.46824,
  0.47019,
  0.47214,
  0.47409,
  0.47604,
  0.47799,
  0.47994,
  0.48189,
  0.48384,
  0.48579,
  0.48775,
   0.4897,
  0.49165,
   0.4936,
  0.49555,
   0.4975,
};
            #endregion
            internal CopperColormapLUT()
                : base(CopperColormapLUT.R, CopperColormapLUT.G, CopperColormapLUT.B) { }
        }
        internal class HSVColormapLUT : ColormapLUT
        {
            #region Colormap Definitions
            static double[] R = new double[]{
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
  0.99219,
  0.96875,
  0.94531,
  0.92188,
  0.89844,
    0.875,
  0.85156,
  0.82813,
  0.80469,
  0.78125,
  0.75781,
  0.73438,
  0.71094,
   0.6875,
  0.66406,
  0.64063,
  0.61719,
  0.59375,
  0.57031,
  0.54688,
  0.52344,
      0.5,
  0.47656,
  0.45313,
  0.42969,
  0.40625,
  0.38281,
  0.35938,
  0.33594,
   0.3125,
  0.28906,
  0.26563,
  0.24219,
  0.21875,
  0.19531,
  0.17188,
  0.14844,
    0.125,
  0.10156,
 0.078125,
 0.054688,
  0.03125,
0.0078125,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
0.0078125,
  0.03125,
 0.054688,
 0.078125,
  0.10156,
    0.125,
  0.14844,
  0.17188,
  0.19531,
  0.21875,
  0.24219,
  0.26563,
  0.28906,
   0.3125,
  0.33594,
  0.35938,
  0.38281,
  0.40625,
  0.42969,
  0.45313,
  0.47656,
      0.5,
  0.52344,
  0.54688,
  0.57031,
  0.59375,
  0.61719,
  0.64063,
  0.66406,
   0.6875,
  0.71094,
  0.73438,
  0.75781,
  0.78125,
  0.80469,
  0.82813,
  0.85156,
    0.875,
  0.89844,
  0.92188,
  0.94531,
  0.96875,
  0.99219,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
};
            static double[] G = new double[]{
       0,
0.023438,
0.046875,
0.070313,
 0.09375,
 0.11719,
 0.14063,
 0.16406,
  0.1875,
 0.21094,
 0.23438,
 0.25781,
 0.28125,
 0.30469,
 0.32813,
 0.35156,
   0.375,
 0.39844,
 0.42188,
 0.44531,
 0.46875,
 0.49219,
 0.51563,
 0.53906,
  0.5625,
 0.58594,
 0.60938,
 0.63281,
 0.65625,
 0.67969,
 0.70313,
 0.72656,
    0.75,
 0.77344,
 0.79688,
 0.82031,
 0.84375,
 0.86719,
 0.89063,
 0.91406,
  0.9375,
 0.96094,
 0.98438,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
 0.97656,
 0.95313,
 0.92969,
 0.90625,
 0.88281,
 0.85938,
 0.83594,
  0.8125,
 0.78906,
 0.76563,
 0.74219,
 0.71875,
 0.69531,
 0.67188,
 0.64844,
   0.625,
 0.60156,
 0.57813,
 0.55469,
 0.53125,
 0.50781,
 0.48438,
 0.46094,
  0.4375,
 0.41406,
 0.39063,
 0.36719,
 0.34375,
 0.32031,
 0.29688,
 0.27344,
    0.25,
 0.22656,
 0.20313,
 0.17969,
 0.15625,
 0.13281,
 0.10938,
0.085938,
  0.0625,
0.039063,
0.015625,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
};
            static double[] B = new double[]{     
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
0.015625,
0.039063,
  0.0625,
0.085938,
 0.10938,
 0.13281,
 0.15625,
 0.17969,
 0.20313,
 0.22656,
    0.25,
 0.27344,
 0.29688,
 0.32031,
 0.34375,
 0.36719,
 0.39063,
 0.41406,
  0.4375,
 0.46094,
 0.48438,
 0.50781,
 0.53125,
 0.55469,
 0.57813,
 0.60156,
   0.625,
 0.64844,
 0.67188,
 0.69531,
 0.71875,
 0.74219,
 0.76563,
 0.78906,
  0.8125,
 0.83594,
 0.85938,
 0.88281,
 0.90625,
 0.92969,
 0.95313,
 0.97656,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
 0.98438,
 0.96094,
  0.9375,
 0.91406,
 0.89063,
 0.86719,
 0.84375,
 0.82031,
 0.79688,
 0.77344,
    0.75,
 0.72656,
 0.70313,
 0.67969,
 0.65625,
 0.63281,
 0.60938,
 0.58594,
  0.5625,
 0.53906,
 0.51563,
 0.49219,
 0.46875,
 0.44531,
 0.42188,
 0.39844,
   0.375,
 0.35156,
 0.32813,
 0.30469,
 0.28125,
 0.25781,
 0.23438,
 0.21094,
  0.1875,
 0.16406,
 0.14063,
 0.11719,
 0.09375,
0.070313,
0.046875,
0.023438,
};
            #endregion
            internal HSVColormapLUT()
                : base(HSVColormapLUT.R, HSVColormapLUT.G, HSVColormapLUT.B) { }
        }
        internal class BinaryColormapLUT : ColormapLUT
        {
            #region Colormap Definitions
            static double[] R = new double[]{
        0,
        1,
};
            static double[] G = new double[]{
       0,
       1
};
            static double[] B = new double[]{     
       0,
       1
};
            #endregion
            internal BinaryColormapLUT()
                : base(BinaryColormapLUT.R, BinaryColormapLUT.G, BinaryColormapLUT.B) { }
        }
        #endregion
    }

}
