﻿using System.IO;
using System.Runtime.Serialization;
using NUnit.Framework;
using Vts.Common;

namespace Vts.Test.Common
{
    [TestFixture]
    public class LongRangeTests
    {

        [Test]
        public void validate_LongRange_constructor_assigns_correct_values()
        {
            var r = new LongRange(0, 9, 10);

            Assert.AreEqual(r.Start, 0L);
            Assert.AreEqual(r.Stop, 9L);
            Assert.AreEqual(r.Delta, 1L);
            Assert.AreEqual(r.Count, 10);
        }

        [Test]
        public void validate_LongRange_default_constructor_assigns_correct_values()
        {
            var r = new LongRange();

            Assert.AreEqual(r.Start, 0L);
            Assert.AreEqual(r.Stop, 1L);
            Assert.AreEqual(r.Delta, 1L);
            Assert.AreEqual(r.Count, 2);
        }

        [Test]
        public void validate_class_is_serializable()
        {
            Assert.IsNotNull(Clone(new LongRange()));
        }

        [Test]
        public void validate_deserialized_class_is_correct()
        {
            var r = new LongRange(0L, 9L, 10);

            var deserializedR = Clone(r);

            Assert.IsNotNull(deserializedR);

            Assert.AreEqual(deserializedR.Start, 0L);
            Assert.AreEqual(deserializedR.Stop, 9L);
            Assert.AreEqual(deserializedR.Delta, 1L);
            Assert.AreEqual(deserializedR.Count, 10);
        }

        private static T Clone<T>(T myObject)
        {
            using (MemoryStream ms = new MemoryStream(1024))
            {
                var dcs = new DataContractSerializer(typeof(T));
                dcs.WriteObject(ms, myObject);
                ms.Seek(0, SeekOrigin.Begin);
                return (T)dcs.ReadObject(ms);
            }
        }
    }
}
